/*
bthash.h Version 0.1.0  Bthash header file
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Windows version */

#include <windows.h>
#include <stdio.h>

#define EXPORT extern "C" __declspec (dllexport)

#define BT_SIGNATURE "xOoCKftE5hjUXeGB"
#define H_SIGNATURE  "BUHJnOdAh6VgimtY"
#define ROOT   256
#define HDRLEN 256
#define MAXINT   0xffffffff
#define UINT_MAX 0xffffffff
#define LOC_MAX  0xffffffff
#define DELETED  255
#define RND_BYTE 15
#define MAX_MSG_SZ  1024
/* Windows:  bit 29 of error code */
#define USR_ERR_BIT  0x10000000
#define BTREE_ERR  300000000
#define HASH_ERR   350000000
#define SYS_ERR    400000000
/* open read only or read/write */
#define READ 0
#define UPDT 1
/* btisrt */
#define ISRTGOOD 0
#define ISRTDUP  1
/* btupdt */
#define UPDTGOOD 0
#define UPDTERR  1
/* btget */
/* btgetnext */
/* btgetprev */
#define FOUND 0
#define NOTFOUND 1
#define BTEOF 2
#define BTBOF 3
#define LEAFNODE 4
#define INDXNODE 5
#define MATCHED 6
#define FRSTPAPA 7
/* btdlet */
#define DLETGOOD 0
#define DLETERR  1
/* btdbck */
#define DBCKGOOD 0
/* btsweep */
#define SWPNEXT 0
#define SWPFRST 1
#define SWPFOUND 0
#define SWPEOF 1
#define SWPERR 2
/* btinit options */
/* truncate old file & initialize */
#define INITOLD 1
/* create new file & initialize */
#define INITNEW 2
/* truncate old file & initialize */
/* or create new file & initialize */
#define INITALL 3

typedef struct bt_nodefmt {
   long left;
   } bt_nodefmt;

typedef struct h_nodefmt {
   long next;
   } h_nodefmt;

typedef struct blkfmt {
   long right;
   int count;
   } blkfmt;

typedef struct stkfmt {
   struct stkfmt *prev;
   struct stkfmt *next;
   long ofst;
   } stkfmt;

typedef struct btreefmt {
   char signature[16];
   char sig_end[4];
   int blksz;
   int nodesz;   /* records per node */
   int rcdlen;
   int keylen;
   int keyofst;
   int dletofst;
   int blkhdr;
   int rcdhdr;
   long nxtfreeblk;
   int rootofst;
   HANDLE hndl;
   int rcdnode;
   int blknode;
   int midofst;
   int seq;
   int rwflag;
   int reason;
   int rcdnum;  /* record number in block */
   unsigned char *rcd;  /* record address */
   unsigned char *key;  /* key    address */
   unsigned char *dletbyte;  /* delete byte address */
   long ofst;
   long ofstleft;
   long ofstrght;
   stkfmt *head;
   stkfmt *tail;
   bt_nodefmt *node;
   blkfmt *blk;
   blkfmt *blkleft;
   blkfmt *blkrght;
   char dbname[128];
   } btreefmt;

typedef struct hashfmt {
   char signature[16];
   char sig_end[4];
   int totrcds;
   int rcdlen;
   int keylen;
   int keyofst;
   int rcdhdr;
   int hdrlen;
   long freestk;
   HANDLE hndl;
   unsigned char *rcd;
   unsigned char *key;
   int rcdnode;
   int reason;
   int seq;
   int fsofst;  /* offset of freestk in header */
   long hashofst;
   long hashend;
   long dbend;
   long ofst;
   long ofstnext;
   long ofstprev;
   h_nodefmt *node;
   h_nodefmt *nodenext;
   h_nodefmt *nodeprev;
   char dbname[128];
   int rwflag;
   } hashfmt;

/********************************************************************/
/*                     btree subroutines                            */
/********************************************************************/

int  btclose(btreefmt *bthdr);

int  btcloset(btreefmt *bthdr);

int  btdbck(btreefmt *bthdr);

void btdbsz(btreefmt *bthdr, int *blocks,
	int *count);

int  btdepth(btreefmt *bthdr);

int  btdlet(btreefmt *bthdr, long ofst,
   unsigned char *key);

void btdlets(btreefmt *bthdr, int *blocks,
        int *count, int *leaf, int *nonleaf,
        int *active, int *deletes,
        int *indxrcd, int *leafrcd);

void btflustk(btreefmt *bthdr);

int  btget(btreefmt *bthdr, long ofst,
	unsigned char *key, unsigned char *buf);

void btgetblk(btreefmt *bthdr, long ofst);

int  btgetnxt(btreefmt *bthdr, long ofst,
	unsigned char *key, unsigned char *buf);

int  btgetp(btreefmt *bthdr, long ofst,
	unsigned char *key, unsigned char *buf);

int  btgetprv(btreefmt *bthdr, long ofst,
	unsigned char *key, unsigned char *buf);

void bthxdmp(btreefmt *bthdr, char *tag);

void bthxmem(int len, unsigned char *mem);

int  btinit(char *dbname, int truncflag, int nodesz,
        int rcdlen, int keylen,
        int keyofst, int dletofst);

int  btisrt(btreefmt *bthdr, long ofst, unsigned char *rcd);

long btnewblk(btreefmt *bthdr);

btreefmt *btopen(char *dbname, int rwflag, int *reason);

btreefmt *btopent(int nodesz, int rcdlen,
   int keylen, int keyofst, int dletofst,
   int *reason, unsigned char *rndSeed);

void btoptim(int totrcds, int nodesz,
	int *levels, int *lvltot,
	int *lvlrem, int *lvlblks,
	int *blktot);

void btpop(btreefmt *bthdr);

void btpush(btreefmt *bthdr, long ofst);

void btputblk(btreefmt *bthdr, long ofst, blkfmt *blk);

void btputhx(int adr, int len, unsigned char *str);

int  btreorg(btreefmt *bthdr);

void btrevers(btreefmt *bthdr, long ofst);

DWORD btseek(btreefmt *bthdr, long ofst);

void btsplit(btreefmt *bthdr);

void btspltrt(btreefmt *bthdr);

void btshwstk(btreefmt *bthdr);

int  btstkdp(btreefmt *bthdr);

stkfmt *btstkini(btreefmt *bthdr);

int  btsweep(btreefmt *bthdr, int position,
	unsigned char *buf);

void bttravgt(btreefmt *bthdr, long ofst, blkfmt *blk);

void bttravrs(btreefmt *bthdr, long ofst);

DWORD bttravsk(btreefmt *bthdr, long ofst);

int btupdt(btreefmt *bthdr, long ofst, unsigned char *rcd);

/********************************************************************/
/*                      hash subroutines                            */
/********************************************************************/

void getmd5(unsigned char *key,
   int keylen, unsigned char *digest);

long hash(hashfmt *hshhdr, unsigned char *key);

int hchain(hashfmt *hshhdr, int *count,
   int *smlen, int *smlensq, int *freelen);

int hclose(hashfmt *hshhdr);

int hcloset(hashfmt *hshhdr);

int hdbck(hashfmt *hshhdr);

int hdlet(hashfmt *hshhdr, unsigned char *key);

int hget(hashfmt *hshhdr, unsigned char *key,
   unsigned char *buf);

void hgetblk(hashfmt *hshhdr, long ofst);

long hgetptr(hashfmt *hshhdr, long ofst);

void hhxdmp(hashfmt *hshhdr, char *tag);

void hhxmem(int len, unsigned char *memory);

int hinit(char *dbname, int truncflag, int nodesz,
	int rcdlen, int keylen, int keyofst);

int hisrt(hashfmt *hshhdr, unsigned char *rcd);

long hnewblk(hashfmt *hshhdr);

hashfmt *hopen(char *dbname, int rwflag,
   int *reason);

hashfmt *hopent(int nodesz, int rcdlen,
   int keylen, int keyofst,
   int *reason, unsigned char *seed);

int hprime(int maxnum);

void hputblk(hashfmt *hshhdr, long ofst, h_nodefmt *blk);

void hputhx(int adr, int len, unsigned char *str);

/* 3rd parameter "pointer" is passed by value */
void hputptr(hashfmt *hshhdr, long ofst, long pointer);

int hreorg(hashfmt *hshhdr, int totrcds,
   double freespc);

long hseek(hashfmt *hshhdr, long ofst);

void hsize(hashfmt *hshhdr, int *rcds,
   int *ovfl, int *empty, int *freeblk);

int hsweep(hashfmt *hshhdr, int position,
   unsigned char *buf);

int hupdt(hashfmt *hshhdr, unsigned char *rcd);

/********** static unsigned char *rndSeed; *****************/

int rnd(unsigned char *rndSeed);

unsigned char *rndinit(void);

unsigned int rndnum(unsigned int lmt, unsigned char *rndSeed);

unsigned int rndpwr(int bits, unsigned char *rndSeed);
