/*
btdlet.c Version 0.1.0  Btree delete record
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* return code 0 is good delete */
/* return code 1 is record not found */
/* return code 3 is lock pending prior to delete */
/* return code 4 is lock error */

#include <bthash.h>

int btdlet(btreefmt *bthdr, long ofst,
   unsigned char *key)
   {
   int sigrslt;
   int btgrslt;
   unsigned char *p,*q,*r;
   unsigned char *buf;
   bthdr->reason = 0;
   sigrslt = strcmp(bthdr->signature,BT_SIGNATURE);
   if (sigrslt)
      {
      fprintf(stderr,"btdlet: non-btree "
         "deletion\n");
      bthdr->reason = BTREE_ERR + 705;
      return(BTREE_ERR + 705);
      } /* not btree database */
   if (ofst == ROOT
      && bthdr->head->next != bthdr->tail)
      {
      fprintf(stderr,"btdlet: deleting at root "
	 "with non-empty stack\n");
      return(BTREE_ERR + 701);
      } /* insert loop */
   buf = (unsigned char *) malloc(bthdr->rcdlen + 16);
   if (buf == NULL)
      {
      fprintf(stderr,"btdlet: out of memory\n");
      return(BTREE_ERR + 702);
      } /* out of mem */
   btgrslt = btget(bthdr,ofst,key,buf);
   free(buf);
   if (btgrslt == FRSTPAPA) return(DLETERR);
   else if (btgrslt) return(btgrslt);
   p = (unsigned char *) bthdr->blk + bthdr->blkhdr
      + (bthdr->rcdnum * bthdr->rcdnode)
      - bthdr->rcdnode + bthdr->rcdhdr;
   if (*(p + bthdr->dletofst) == DELETED)
      {
      fprintf(stderr,"btdlet: deleting twice\n");
      bthdr->reason = BTREE_ERR + 704;
      return(BTREE_ERR + 704);
      } /* deleting twice */
   if (bthdr->blk->right)   /* delete index node */
      *(p + bthdr->dletofst) = DELETED;
   else if (bthdr->rcdnum >= bthdr->blk->count)
      {
      p = (unsigned char *) bthdr->blk + bthdr->blkhdr
         + (bthdr->rcdnum * bthdr->rcdnode)
         - bthdr->rcdnode;
      q = (unsigned char *) p + bthdr->rcdnode;
      while (p < q) *p++ = '\0';
      bthdr->blk->count--;
      } /* delete last record in leaf node */
   else
      {
      /* shift left 1 record */
      p = (unsigned char *) bthdr->blk + bthdr->blkhdr
         + (bthdr->rcdnum * bthdr->rcdnode)
         - bthdr->rcdnode;
      q = (unsigned char *) bthdr->blk + bthdr->blksz;
      r = (unsigned char *) p + bthdr->rcdnode;
      while (r < q) *p++ = *r++;
      p = (unsigned char *) bthdr->blk + bthdr->blkhdr
         + (bthdr->blk->count * bthdr->rcdnode);
      q = (unsigned char *) bthdr->blk + bthdr->blksz;
      while (p < q) *p++ = '\0';
      bthdr->blk->count--;
      } /* delete leaf node */
   btputblk(bthdr,bthdr->ofst,bthdr->blk);
   if (bthdr->reason) return(BTREE_ERR + 703);
   p = (unsigned char *) bthdr->blk;
   q = (unsigned char *) bthdr->blk + bthdr->blksz;
   while (p < q) *p++ = '\0';
   bthdr->ofst = (long) 0;
   return(DLETGOOD);
   } /* btdlet */
